﻿using System;
using System.Drawing;
using System.Windows;
using System.Windows.Forms;
using System.Windows.Resources;
using Application = System.Windows.Application;

namespace LEDControl
{
    /// <summary>
    /// Interaction logic for App.xaml
    /// </summary>
    public partial class App : Application
    {
        public App()
        {
            _ledDevice = new LEDDevice();
            _ledApplicationsModel = LEDApplicationsModel.Create();
        }

        private void Application_Startup(object sender, StartupEventArgs e)
        {

            _notifyIcon = new NotifyIcon();
            StreamResourceInfo sri = GetResourceStream(new Uri("pack://application:,,,/LEDControl;component/Images/AppIcon.ico"));
            if (sri != null)
                _notifyIcon.Icon = new Icon(sri.Stream);
            _notifyIcon.Visible = true;
            _notifyIcon.Text = "LED Control";
            MenuItem openMenu = new MenuItem("Open");
            openMenu.Click += openMenu_Click;
            MenuItem exitMenu = new MenuItem("Exit");
            exitMenu.Click += exitMenu_Click;
            _notifyIcon.ContextMenu = new ContextMenu(
                new [] {openMenu, exitMenu });
            _notifyIcon.DoubleClick += openMenu_Click;

            Runner.Create(_ledApplicationsModel.RootApplication, _ledDevice);
        }

        void openMenu_Click(object sender, EventArgs e)
        {
            if (_mainWindow == null)
            {
                _mainWindow = new MainWindow();
                _mainWindow.DataContext = _ledApplicationsModel;
                _mainWindow.Closed += delegate { _mainWindow = null; };
            }
            _mainWindow.Show();
            _mainWindow.WindowState = WindowState.Normal;
        }

        void exitMenu_Click(object sender, EventArgs e)
        {
            Shutdown(0);
        }

        private void Application_Exit(object sender, ExitEventArgs e)
        {
            if (_ledDevice != null)
            {
                _ledDevice.Write("");
                _ledDevice.SetIndicators(new Indicators());
            }
            _notifyIcon.Visible = false;
        }

        private NotifyIcon _notifyIcon;
        private MainWindow _mainWindow;
        private readonly LEDDevice _ledDevice;
        private readonly LEDApplicationsModel _ledApplicationsModel;
    }
}
