﻿using System;
using System.Windows.Forms;

namespace LEDControl
{
    public delegate void DeviceChangedHandler(IntPtr wParam, IntPtr lParam);

    [System.Security.Permissions.PermissionSet(System.Security.Permissions.SecurityAction.Demand, Name = "FullTrust")]
    class DeviceChangedWatcher : NativeWindow
    {
        public event DeviceChangedHandler DeviceChanged;

        // Constant values were found in the "windows.h" header file.
        private const int WM_DEVICECHANGE = 0x219;

        public DeviceChangedWatcher()
        {
            CreateParams cp = new CreateParams
                              {
                                  // Fill in the CreateParams details.
                                  Caption = "LEDControl Message Receiver",
                                  ClassName = "Button",

                                  // Set the position on the form
                                  X = 100,
                                  Y = 100,
                                  Height = 100,
                                  Width = 100
                              };

            // Create the actual window
            CreateHandle(cp);
        }

        [System.Security.Permissions.PermissionSet(System.Security.Permissions.SecurityAction.Demand, Name = "FullTrust")]
        protected override void WndProc(ref Message m)
        {
            // Listen for messages that are sent to the button window. Some messages are sent
            // to the parent window instead of the button's window.
            switch (m.Msg)
            {
                case WM_DEVICECHANGE:
                    if (DeviceChanged != null)
                        DeviceChanged(m.WParam, m.LParam);
                    break;
            }
            base.WndProc(ref m);
        }
    }
}
