﻿using System;
using System.Linq;
using System.Windows;
using System.Windows.Threading;

namespace LEDControl
{
    public class LEDApplicationLauncher : LEDApplicationSelector
    {
        private int _button1PressedAt;
        private bool _firstActivation;

        public override bool Start(LEDApplicationTree tree, LifecycleArgs args)
        {
            base.Start(tree, args);
            _firstActivation = true;
            return (tree.Children != null && tree.Children.Count > 0);
        }

        public override void Activate(LEDApplicationTree tree, LifecycleArgs args)
        {
            base.Activate(tree, args);
            // If Button 1 was just pressed
            if (Environment.TickCount - _button1PressedAt < 50)
            {
                Application.Current.Dispatcher.BeginInvoke(
                    (Action)(() => RequestTerminate(args.Runner)), DispatcherPriority.Normal);
            }
            else if (_firstActivation)
            {
                _firstActivation = false;
                if (tree.Children.Count > 0)
                    RequestExecute(args.Runner, tree.Children[0]);
            }
        }

        public override void OnPreviewButtonDown(LEDApplicationTree node, LEDDeviceButtonEventArgs args)
        {
            base.OnPreviewButtonDown(node, args);
            if (args.Button == Buttons.Button1)
            {
                _button1PressedAt = Environment.TickCount;
            }
        }

        public override void OnButtonDown(LEDApplicationTree node, LEDDeviceButtonEventArgs args)
        {
            // If the running application is our immediate child
            if (node.Children.Select(item => item.Application).Contains(args.Runner.CurrentlyRunning))
            {
                int buttonBits = (int) args.Button;
                foreach (LEDApplicationTree child in node.Children)
                {
                    buttonBits >>= 1;
                    if ((buttonBits & 1) != 0)
                    {
                        args.Runner.CurrentlyRunning.RequestTerminate(args.Runner);
                        RequestExecute(args.Runner, child);
                        args.Handled = true;
                        break;
                    }
                }
            }
        }

        protected override int GetMaxChildren()
        {
            return 7;
        }
    }
}
