﻿
using System;
using System.Windows;
using System.Windows.Threading;
using System.Xml.Serialization;
using System.Linq;

namespace LEDControl
{
    public abstract class LEDApplicationSelector : LEDApplication
    {
        private bool _firstActivation;

        public override bool Start(LEDApplicationTree tree, LifecycleArgs args)
        {
            _firstActivation = true;
            bool success = base.Start(tree, args);
            _timer = new DispatcherTimer(DispatcherPriority.Normal, Application.Current.Dispatcher);
            _timer.Tick += delegate { OnTimer(tree, args.Runner); };

            return success;
        }

        public override void Activate(LEDApplicationTree tree, LifecycleArgs args)
        {
            base.Activate(tree, args);
            if (_firstActivation)
            {
                _firstActivation = false;
                Application.Current.Dispatcher.BeginInvoke(
                    (Action) (() => OnTimer(tree, args.Runner)), DispatcherPriority.Normal);
            }
        }

        public override void Terminate(LEDApplicationTree tree, LifecycleArgs args)
        {
            base.Terminate(tree, args);
            _timer.Stop();
            _timer = null;
        }

        private void OnTimer(LEDApplicationTree tree, Runner runner)
        {
            if (IsStarted && Cycle && !CycleTime.IsZero)
            {
                // If the running app is one of our children
                int index = tree.Children.Select(item => item.Application).ToList().IndexOf(runner.CurrentlyRunning);
                if (index != -1)
                {
                    // Terminate it
                    runner.CurrentlyRunning.RequestTerminate(runner);
                    // Find the next child
                    if (++index >= tree.Children.Count)
                        index = 0;
                }
                // Else if we're the running app
                else if (runner.CurrentlyRunning == this)
                    index = 0;

                if (index >= 0 && index < tree.Children.Count)
                    RequestExecute(runner, tree.Children[index]);

                _timer.Interval = CycleTime.ToTimeSpan();
                _timer.Start();
            }
            else
                _timer.Stop();
        }

        #region Fields

        private DispatcherTimer _timer;

        #endregion

        #region Properties

        private bool _cycle;
        [XmlAttribute]
        public bool Cycle
        {
            get { return _cycle; }
            set
            {
                _cycle = value; 
                OnPropertyChanged("Cycle");
                OnCycleSettingsChanged();
            }
        }

        private TimeCountUnits _cycleTime = new TimeCountUnits {Count = 5.0, Units = "Seconds"};
        public TimeCountUnits CycleTime
        {
            get { return _cycleTime; }
            set
            {
                _cycleTime = value; 
                OnPropertyChanged("CycleTime");
                OnCycleSettingsChanged();
            }
        }

        #endregion

        private void OnCycleSettingsChanged()
        {
            if (_timer != null)
            {
                if (Cycle && !CycleTime.IsZero)
                {
                    _timer.Interval = new TimeSpan(1);
                    _timer.Start();
                }
                else
                    _timer.Stop();
            }
        }
    }
}
