﻿using System;
using System.IO;
using System.Xml.Serialization;
using System.ComponentModel;

namespace LEDControl
{
    public class LEDApplicationsModel : INotifyPropertyChanged
    {
        public static LEDApplicationsModel Create()
        {
            LEDApplicationsModel ledApplicationsModel = null;

            if (File.Exists(modelFileName))
            {
                try
                {
                    XmlSerializer xs = new XmlSerializer(typeof(LEDApplicationsModel));
                    using (Stream s = File.OpenRead(modelFileName))
                    {
                        ledApplicationsModel = (LEDApplicationsModel) xs.Deserialize(s);
                    }
                    ledApplicationsModel.IsDirty = false;
                }
                catch {}
            }

            if (ledApplicationsModel == null)
            {
                ledApplicationsModel = new LEDApplicationsModel();
                LEDApplication startUpApp = LEDApplication.Create(LEDApplication.ApplicationTypes[0], "Start Up");
                LEDApplicationTree rootNode = new LEDApplicationTree(startUpApp);
                ledApplicationsModel.RootApplication = rootNode;
                ledApplicationsModel.IsDirty = true;
            }

            return ledApplicationsModel;
        }

        public void Save()
        {
            XmlSerializer xs = new XmlSerializer(typeof(LEDApplicationsModel));
            using (Stream s = File.Create(modelFileName))
            {
                xs.Serialize(s, this);
            }
            IsDirty = false;
        }

        #region Properties

        private LEDApplicationTree _rootApplication;
        public LEDApplicationTree RootApplication
        {
            get { return _rootApplication; }
            set
            {
                _rootApplication = value;
                RootApplication.PropertyChanged += RootApplication_PropertyChanged;
                OnPropertyChanged("RootApplication");
            }
        }

        private bool _isDirty;

        [XmlIgnore]
        public bool IsDirty
        {
            get { return _isDirty; }
            set
            {
                _isDirty = value;
                OnPropertyChanged("IsDirty");
                Title = "LED Control" + (IsDirty ? " *" : "");
            }
        }

        private string _title;

        [XmlIgnore]
        public string Title
        {
            get { return _title; }
            set { _title = value; OnPropertyChanged("Title"); }
        }

        static readonly string modelFileName =
            Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData),
                                   Path.Combine("Alexander_Software", "LEDControl.xml"));

        #endregion

        #region Event Handlers

        void RootApplication_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            IsDirty = true;
        }

        #endregion

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;
        void OnPropertyChanged(string name)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(name));
        }

        #endregion
    }
}
