﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Diagnostics;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;

namespace LEDControl
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private LEDApplicationsModel _viewModel;

        public MainWindow()
        {
            InitializeComponent();
        }

        #region Command Handlers

        private void save_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            ((LEDApplicationsModel) e.Parameter).Save();
        }

        private void save_CanExecute(object sender, CanExecuteRoutedEventArgs e)
        {
            e.CanExecute = (e.Parameter is LEDApplicationsModel) && ((LEDApplicationsModel) e.Parameter).IsDirty;
        }

        private void exit_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            Application.Current.Shutdown(0);
        }

        private void helpContents_Executed(object sender, ExecutedRoutedEventArgs e)
        {
        }

        private void about_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            MessageBox.Show("LEDControl\n\nCopyright © 2011 by Robert E. Alexander", "About");
        }

        private void addDataItem_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            LEDApplicationDataViewer dataViewer = (LEDApplicationDataViewer) e.Parameter;
            dataViewer.AddDataItem();
        }

        private void addDataItem_CanExecute(object sender, CanExecuteRoutedEventArgs e)
        {
            e.CanExecute = true;
        }

        private void deleteDataItem_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            if (e.Parameter is IList<object>)
            {
                IList<object> values = (IList<object>) e.Parameter;
                ((LEDApplicationDataViewer) values[1]).DeleteDataItem((LEDApplicationDataViewer.DataItem) values[0]);
            }
        }

        private void deleteDataItem_CanExecute(object sender, CanExecuteRoutedEventArgs e)
        {
            e.CanExecute = true;
        }

        private void testDataViewer_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            IList<object> objects = e.Parameter as IList<object>;
            if (objects != null && objects.Count >= 1)
            {
                LEDApplicationDataViewer dataViewer = objects[0] as LEDApplicationDataViewer;
                if (dataViewer != null)
                    dataViewer.Test();
            }
        }

        private void testDataViewer_CanExecute(object sender, CanExecuteRoutedEventArgs e)
        {
            e.CanExecute = true;
        }

        #endregion

        #region Event Handlers

        private void MainWindow_Loaded(object sender, RoutedEventArgs e)
        {
            _viewModel.IsDirty = false;
        }

        private void MainWindow_DataContextChanged(object sender, DependencyPropertyChangedEventArgs e)
        {
            _viewModel = DataContext as LEDApplicationsModel;
            _menuTreeView.DataContext = new LEDApplicationViewModel(_viewModel);
        }

        private void MainWindow_Closing(object sender, CancelEventArgs e)
        {
            if (_viewModel.IsDirty)
            {
                MessageBoxResult answer =
                    MessageBox.Show("You have made changes to the applications. Do you want to save them?",
                                    "Confirm Changes", MessageBoxButton.YesNoCancel);
                if (answer == MessageBoxResult.Yes)
                    _viewModel.Save();
                else if (answer == MessageBoxResult.Cancel)
                    e.Cancel = true;
            }
        }

        /// <summary>
        /// Called when the user changes the text of a DataItem property value. The TextBox's
        /// DataContext is a Parameter and its Tag is a Dictionary of parameter names to values
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PropertyValue_TextChanged(object sender, TextChangedEventArgs e)
        {
            TextBox textBox = (TextBox) sender;
            DataItemRetriever.Parameter parameter = (DataItemRetriever.Parameter) textBox.DataContext;
            ObservableCollection<LEDApplicationDataViewer.DataItem.ParameterValue> parameterValues =
                (ObservableCollection<LEDApplicationDataViewer.DataItem.ParameterValue>)textBox.Tag;
            LEDApplicationDataViewer.DataItem.ParameterValue pv = parameterValues.FirstOrDefault(item => item.ParameterName == parameter.Name);
            if (pv != null)
                pv.Value = textBox.Text;
            else
            {
                parameterValues.Add(new LEDApplicationDataViewer.DataItem.ParameterValue
                                    {
                                        ParameterName = parameter.Name,
                                        Value = textBox.Text
                                    });
            }
        }

        private void PropertyValue_TimeCountUnitsChanged(object sender, RoutedEventArgs e)
        {
            TimeCountUnitsControl tcuc = (TimeCountUnitsControl) sender;
            DataItemRetriever.Parameter parameter = (DataItemRetriever.Parameter)tcuc.DataContext;
            ObservableCollection<LEDApplicationDataViewer.DataItem.ParameterValue> parameterValues =
                (ObservableCollection<LEDApplicationDataViewer.DataItem.ParameterValue>)tcuc.Tag;
            LEDApplicationDataViewer.DataItem.ParameterValue pv = parameterValues.FirstOrDefault(item => item.ParameterName == parameter.Name);
            if (pv != null)
                pv.Value = tcuc.TimeCountUnits;
            else
            {
                parameterValues.Add(new LEDApplicationDataViewer.DataItem.ParameterValue
                {
                    ParameterName = parameter.Name,
                    Value = tcuc.TimeCountUnits
                });
            }
        }

        private void IndicatorExpression_TextChanged(object sender, TextChangedEventArgs e)
        {
            TextBox textBox = (TextBox) sender;
            LEDApplicationDataViewer.Expression expression = (LEDApplicationDataViewer.Expression) textBox.DataContext;
            expression.Text = textBox.Text;
        }

        private void ActionExpression_TextChanged(object sender, TextChangedEventArgs e)
        {
            TextBox textBox = (TextBox)sender;
            LEDApplicationDataViewer.Expression expression = (LEDApplicationDataViewer.Expression)textBox.DataContext;
            if (expression != null)
                expression.Text = textBox.Text;
        }

        private void Hyperlink_Click(object sender, RoutedEventArgs e)
        {
            Hyperlink hyperlink = (Hyperlink) sender;
            Process.Start(hyperlink.NavigateUri.AbsoluteUri);
        }

        #endregion
    }
}
