﻿using System;
using System.Globalization;
using System.Windows;
using System.Windows.Controls;
using System.ComponentModel;
using System.Windows.Data;
using System.Xml.Serialization;

namespace LEDControl
{
    public class TimeCountUnits : INotifyPropertyChanged
    {
        private double _count;
        public double Count
        {
            get { return _count; }
            set
            {
                _count = value;
                OnPropertyChanged("Count");
            }
        }

        private string _units = TimeCountUnitsControl.UnitNames[0];
        public string Units
        {
            get { return _units; }
            set
            {
                _units = value;
                OnPropertyChanged("Units");
            }
        }

        [XmlIgnore]
        public bool IsZero
        {
            get { return _count == 0.0; }
        }

        public TimeSpan ToTimeSpan()
        {
            if (Units == TimeCountUnitsControl.UnitNames[0])
                return new TimeSpan((long) (Count * TimeSpan.TicksPerSecond));
            if (Units == TimeCountUnitsControl.UnitNames[1])
                return new TimeSpan((long)(Count * TimeSpan.TicksPerMinute));
            if (Units == TimeCountUnitsControl.UnitNames[2])
                return new TimeSpan((long)(Count * TimeSpan.TicksPerHour));
            return new TimeSpan();
        }

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;
        void OnPropertyChanged(string name)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(name));
        }

        #endregion
    }

    /// <summary>
    /// Interaction logic for TimeCountUnitsControl.xaml
    /// </summary>
    public partial class TimeCountUnitsControl : UserControl
    {
        public TimeCountUnitsControl()
        {
            InitializeComponent();
        }

        public static string[] UnitNames = {"Seconds", "Minutes", "Hours"};

        public TimeCountUnits TimeCountUnits
        {
            get { return (TimeCountUnits)GetValue(TimeCountUnitsProperty); }
            set { SetValue(TimeCountUnitsProperty, value); }
        }

        // Using a DependencyProperty as the backing store for TimeCountUnits.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty TimeCountUnitsProperty =
            DependencyProperty.Register("TimeCountUnits", typeof(TimeCountUnits), typeof(TimeCountUnitsControl),
            new FrameworkPropertyMetadata(OnTimeCountUnitsChanged));

        private static void OnTimeCountUnitsChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            TimeCountUnitsControl tcuc = (TimeCountUnitsControl) d;
            if (tcuc.TimeCountUnits != null)
                tcuc.TimeCountUnits.PropertyChanged += delegate { tcuc.OnTimeCountUnitsChanged(); };
            tcuc.OnTimeCountUnitsChanged();
        }

        private void OnTimeCountUnitsChanged()
        {
            RaiseEvent(new RoutedEventArgs(TimeCountUnitsChangedEvent, this));
        }

        public static readonly RoutedEvent TimeCountUnitsChangedEvent = EventManager.RegisterRoutedEvent(
            "TimeCountUnitsChanged", RoutingStrategy.Direct, typeof(RoutedEventHandler), typeof(TimeCountUnitsControl));

        public event RoutedEventHandler TimeCountUnitsChanged
        {
            add {AddHandler(TimeCountUnitsChangedEvent, value);}
            remove {RemoveHandler(TimeCountUnitsChangedEvent, value);}
        }
    }

    public class DoubleToStringConverter : IValueConverter
    {
        #region Implementation of IValueConverter

        public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            return value.ToString();
        }

        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            double result;
            double.TryParse(value.ToString(), out result);
            return result;
        }

        #endregion
    }
}
